#define _XOPEN_SOURCE 500
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#define HT2020_DEV0	"/dev/ht2020_100"
#define HT2020_DEV1	"/dev/ht2020_101"
#define HT2020_DEV2	"/dev/ht2020_102"

#define ON_DELAY_MS	(100)
#define OFF_DELAY_MS	(10)

#define CHANNEL_NR	(24)
#define CH_MASK_MAX	(0x1 << (CHANNEL_NR - 1))

static int open_dev(const char *dev_path)
{
	int ret;

	ret = open(dev_path, O_RDWR);
	if (ret < 0) {
		fprintf(stderr, "Error opening %s: %s\n", dev_path,
			strerror(errno));
		return -1;
	}

	return ret;
}

static void x_pwrite_byte(int fd, const void *buf)
{
	int ret;

	ret = pwrite(fd, buf, 1, 0);
	if (ret < 0) {
		fprintf(stderr, "Error writing to device: %s\n", 
			strerror(errno));
		exit(1);
	}
}

static void x_pread_byte(int fd, void *buf)
{
	int ret;

	ret = pread(fd, buf, 1, 0);
	if (ret < 1) {
		fprintf(stderr, "Error reading from device: %s\n", 
			strerror(errno));
		exit(1);
	}
}

int main(int argc, char *argv[])
{
	int fd0, fd1, fd2;
	unsigned int ch_mask;
	unsigned char val[3], off = 0;
	int verbose = 0;

	fd0 = open_dev(HT2020_DEV0);
	fd1 = open_dev(HT2020_DEV1);
	fd2 = open_dev(HT2020_DEV2);
	if (fd0 < 0 || fd1 < 0 || fd2 < 0)
		return 1;

	if (argc > 1 && strcmp(argv[1], "verbose") == 0)
		verbose = 1;

	while (1) {

		for (ch_mask = 0x1; ch_mask <= CH_MASK_MAX; ch_mask <<= 1) {

			val[0] = (ch_mask)       & 0xff;
			val[1] = (ch_mask >> 8)  & 0xff;
			val[2] = (ch_mask >> 16) & 0xff;

			/* on */
			x_pwrite_byte(fd0, &val[0]);
			x_pwrite_byte(fd1, &val[1]);
			x_pwrite_byte(fd2, &val[2]);
			usleep(ON_DELAY_MS * 1000);

			if (verbose) {
				x_pread_byte(fd0, &val[0]);
				x_pread_byte(fd1, &val[1]);
				x_pread_byte(fd2, &val[2]);
				printf("read: 0x%02X:%02X:%02X\n",
					val[2], val[1], val[0]);
			}

			/* off */
			x_pwrite_byte(fd0, &off);
			x_pwrite_byte(fd1, &off);
			x_pwrite_byte(fd2, &off);
			usleep(OFF_DELAY_MS * 1000);

		}

	}

	close(fd0);
	close(fd1);
	close(fd2);

	exit(0);
}

